;;; custom-load.el --- automatically extracted custom dependencies

;;; Code:

(autoload 'custom-add-loads "cus-load")

(custom-add-loads 'mime '("pgg-def"))
(custom-add-loads 'pgg '("pgg-def" "pgg-gpg" "pgg-parse" "pgg-pgp" "pgg-pgp5"))
(custom-add-loads 'pgg-pgp5 '("pgg-pgp5"))
(custom-add-loads 'pgg-pgp '("pgg-pgp"))
(custom-add-loads 'pgg-parse '("pgg-parse"))
(custom-add-loads 'pgg-gpg '("pgg-gpg"))

;;; custom-load.el ends here
